@@ -10,6 +10,7 @@ use rustc_data_structures::fx::{FxHashMap as HashMap, FxHashSet as HashSet};
 use rustc_hir::{BodyId, ItemKind};
 use rustc_middle::ty::TyCtxt;
 use rustc_span::{source_map::FileLoader, BytePos, Span, SyntaxContext};
+use unicode_segmentation::UnicodeSegmentation;
 
 use crate::{
   extensions::{ContextMode, EvalMode, MutabilityMode, PointerMode, EVAL_MODE},
@@ -201,11 +202,16 @@ pub fn color_ranges(prog: &str, all_ranges: Vec<(&str, &HashSet<Range>)>) -> Str
       ranges.iter().flat_map(|range| {
         let contained = all_ranges.iter().any(|(_, ranges)| {
           ranges.iter().any(|other| {
-            range != other && other.start <= range.end && range.end < other.end
+            range != other
+              && other.byte_start <= range.byte_end
+              && range.byte_end < other.byte_end
           })
         });
         let end_marker = if contained { "]" } else { "\x1B[0m]" };
-        [("[\x1B[31m", range.start), (end_marker, range.end)]
+        [
+          ("[\x1B[31m", range.byte_start),
+          (end_marker, range.byte_end),
+        ]
       })
     })
     .collect::<Vec<_>>();
@@ -242,17 +248,17 @@ pub fn compare_ranges(expected: HashSet<Range>, actual: HashSet<Range>, prog: &s
 pub fn bless(path: &Path, contents: String, actual: HashSet<Range>) -> Result<()> {
   let mut delims = actual
     .into_iter()
-    .flat_map(|range| [("`[", range.start), ("]`", range.end)])
+    .flat_map(|range| [("`[", range.char_start), ("]`", range.char_end)])
     .collect::<Vec<_>>();
   delims.sort_by_key(|(_, i)| *i);
 
   let mut output = String::new();
-  for (i, ch) in contents.chars().enumerate() {
+  for (i, g) in contents.graphemes(true).enumerate() {
     while delims.len() > 0 && delims[0].1 == i {
       let (delim, _) = delims.remove(0);
       output.push_str(delim);
     }
-    output.push(ch);
+    output.push_str(g);
   }
 
   fs::write(path.with_extension("txt.expected"), output)?;
@@ -271,10 +277,8 @@ fn parse_range_map(
       (
         k,
         vs.into_iter()
-          .map(|(start, end)| Range {
-            start,
-            end,
-            filename: "dummy.rs".to_string(),
+          .map(|(byte_start, byte_end)| {
+            Range::from_byte_range(byte_start, byte_end, &clean, "dummy.rs".into())
           })
           .collect::<Vec<_>>(),
       )
@@ -333,12 +337,11 @@ pub fn test_command_output(
 
         match expected {
           Some(expected_path) => {
-            let expected_file = fs::read(expected_path);
+            let expected_file = fs::read_to_string(expected_path);
             match expected_file {
               Ok(file) => {
-                let output = String::from_utf8(file).unwrap();
                 let (_output_clean, output_ranges) =
-                  parse_range_map(&output, vec![("`[", "]`")]).unwrap();
+                  parse_range_map(&file, vec![("`[", "]`")]).unwrap();
 
                 let expected = match output_ranges.get("`[") {
                   Some(ranges) => ranges.clone().into_iter().collect::<HashSet<_>>(),
